Stan Math Library: stan/math/prim/scal/prob/normal_ccdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

normal_ccdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_NORMAL_CCDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_NORMAL_CCDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/value_of.hpp>
13 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
14 #include <stan/math/prim/scal/meta/max_size.hpp>
15 #include <boost/random/normal_distribution.hpp>
16 #include <boost/random/variate_generator.hpp>
17 #include <cmath>
18 #include <limits>
19
20 namespace stan {
21
22 namespace math {
23
24 template <typename T_y, typename T_loc, typename T_scale>
25 typename return_type<T_y, T_loc, T_scale>::type
26 normal_ccdf_log(const T_y& y, const T_loc& mu, const T_scale& sigma) {
27 static const char* function("stan::math::normal_ccdf_log");
28 typedef typename stan::partials_return_type<T_y, T_loc, T_scale>::type
29 T_partials_return;
30
31 using stan::math::check_positive;
32 using stan::math::check_finite;
33 using stan::math::check_not_nan;
34 using stan::math::check_consistent_sizes;
35 using stan::math::value_of;
36 using stan::math::INV_SQRT_2;
37 using std::log;
38 using std::exp;
39
40 T_partials_return ccdf_log(0.0);
41 // check if any vectors are zero length
42 if (!(stan::length(y)
43 && stan::length(mu)
44 && stan::length(sigma)))
45 return ccdf_log;
46
47 check_not_nan(function, "Random variable", y);
48 check_finite(function, "Location parameter", mu);
49 check_not_nan(function, "Scale parameter", sigma);
50 check_positive(function, "Scale parameter", sigma);
51 check_consistent_sizes(function,
52 "Random variable", y,
53 "Location parameter", mu,
54 "Scale parameter", sigma);
55
56 OperandsAndPartials<T_y, T_loc, T_scale>
57 operands_and_partials(y, mu, sigma);
58
59 VectorView<const T_y> y_vec(y);
60 VectorView<const T_loc> mu_vec(mu);
61 VectorView<const T_scale> sigma_vec(sigma);
62 size_t N = max_size(y, mu, sigma);
63 double log_half = std::log(0.5);
64
65 const double SQRT_TWO_OVER_PI = std::sqrt(2.0 / stan::math::pi());
66 for (size_t n = 0; n < N; n++) {
67 const T_partials_return y_dbl = value_of(y_vec[n]);
68 const T_partials_return mu_dbl = value_of(mu_vec[n]);
69 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
70
71 const T_partials_return scaled_diff = (y_dbl - mu_dbl)
72 / (sigma_dbl * SQRT_2);
73
74 T_partials_return one_m_erf;
75 if (scaled_diff < -37.5 * INV_SQRT_2)
76 one_m_erf = 2.0;
77 else if (scaled_diff < -5.0 * INV_SQRT_2)
78 one_m_erf = 2.0 - erfc(-scaled_diff);
79 else if (scaled_diff > 8.25 * INV_SQRT_2)
80 one_m_erf = 0.0;
81 else
82 one_m_erf = 1.0 - erf(scaled_diff);
83
84 // log ccdf
85 ccdf_log += log_half + log(one_m_erf);
86
87 // gradients
88 if (contains_nonconstant_struct<T_y, T_loc, T_scale>::value) {
89 const T_partials_return rep_deriv_div_sigma
90 = scaled_diff > 8.25 * INV_SQRT_2
91 ? std::numeric_limits<double>::infinity()
92 : SQRT_TWO_OVER_PI * exp(-scaled_diff * scaled_diff)
93 / one_m_erf / sigma_dbl;
94 if (!is_constant_struct<T_y>::value)
95 operands_and_partials.d_x1[n] -= rep_deriv_div_sigma;
96 if (!is_constant_struct<T_loc>::value)
97 operands_and_partials.d_x2[n] += rep_deriv_div_sigma;
98 if (!is_constant_struct<T_scale>::value)
99 operands_and_partials.d_x3[n] += rep_deriv_div_sigma
100 * scaled_diff * stan::math::SQRT_2;
101 }
102 }
103 return operands_and_partials.value(ccdf_log);
104 }
105 }
106 }
107 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::sqrt
fvar< T > sqrt(const fvar< T > &x)
Definition: sqrt.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
contains_nonconstant_struct.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::erf
fvar< T > erf(const fvar< T > &x)
Definition: erf.hpp:14
is_constant_struct.hpp
stan::math::normal_ccdf_log
return_type< T_y, T_loc, T_scale >::type normal_ccdf_log(const T_y &y, const T_loc &mu, const T_scale &sigma)
Definition: normal_ccdf_log.hpp:26
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::SQRT_2
const double SQRT_2
The value of the square root of 2, .
Definition: constants.hpp:21
stan::math::INV_SQRT_2
const double INV_SQRT_2
The value of 1 over the square root of 2, .
Definition: constants.hpp:27
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::math::check_positive
bool check_positive(const char *function, const char *name, const T_y &y)
Return true if y is positive.
Definition: check_positive.hpp:68
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
partials_return_type.hpp
OperandsAndPartials.hpp
stan::contains_nonconstant_struct
Definition: contains_nonconstant_struct.hpp:14
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::erfc
fvar< T > erfc(const fvar< T > &x)
Definition: erfc.hpp:14
stan::math::pi
double pi()
Return the value of pi.
Definition: constants.hpp:86
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
max_size.hpp

[ Stan Home Page ] © 2011–2016, Stan Development Team.